@@ -26,21 +26,14 @@ import java.util.concurrent.atomic.AtomicInteger
 
 private val htmlMimeTypes = setOf(ContentType.TEXT_HTML.mimeType, ContentType.APPLICATION_XHTML_XML.mimeType)
 
+private val attributeKeys = mapOf(
+    "a[href]" to "abs:href",
+    "img[src]" to "abs:src"
+)
+
 val Throwable.rootMessage: String
     get() = ExceptionUtils.getRootCauseMessage(this)
 
-private fun getChildren(document: Document) = (getHrefChildren(document) + getImgChildren(document)).distinct()
-
-private fun getHrefChildren(document: Document) = getChildren(document, "a[href]", "abs:href")
-
-private fun getImgChildren(document: Document) = getChildren(document, "img[src]", "abs:src")
-
-private fun getChildren(document: Document, cssQuery: String, attributeKey: String): Sequence<SpongeUri> {
-    return document.select(cssQuery)
-        .asSequence()
-        .mapNotNull { toSpongeUri(it, attributeKey) }
-}
-
 private fun toSpongeUri(element: Element, attributeKey: String): SpongeUri? {
     return try {
         element.attr(attributeKey)?.let {
@@ -69,8 +62,8 @@ class Sponge(private val spongeService: SpongeService, private val spongeConfig:
             }.let {
                 downloadOrVisitChildren(it, parents)
             }
-        } catch (e: Exception) {
-            System.err.println("⚠ Processing failed for $spongeUri: ${e.rootMessage}")
+        } catch (t: Throwable) {
+            System.err.println("⚠ Processing failed for $spongeUri: ${t.rootMessage}")
         }
     }
 
@@ -86,7 +79,7 @@ class Sponge(private val spongeService: SpongeService, private val spongeConfig:
             if (htmlMimeTypes.contains(mimeType)) {
                 val document = Jsoup.parse(response.body(), response.url().toExternalForm())
 
-                loadChildren(spongeUri, document)
+                spongeUri.children = getChildren(spongeUri, document)
             } else if (spongeConfig.mimeTypes.contains(mimeType)) {
                 spongeUri.download = true
             }
@@ -115,12 +108,27 @@ class Sponge(private val spongeService: SpongeService, private val spongeConfig:
             .awaitAll()
     }
 
-    private fun loadChildren(spongeUri: SpongeUri, document: Document) {
-        spongeUri.children = getChildren(document)
-            .filter { it != spongeUri && isHostVisitable(it.host) }
+    private fun getChildren(spongeUri: SpongeUri, document: Document): Set<SpongeUri> {
+        return attributeKeys.entries
+            .asSequence()
+            .map { getChildren(spongeUri, document, it.key, it.value) }
+            .flatMap { it }
             .toSet()
     }
 
+    private fun getChildren(
+        spongeUri: SpongeUri,
+        document: Document,
+        cssQuery: String,
+        attributeKey: String
+    ): Sequence<SpongeUri> {
+        return document.select(cssQuery)
+            .asSequence()
+            .distinct()
+            .mapNotNull { toSpongeUri(it, attributeKey) }
+            .filter { it != spongeUri && isHostVisitable(it.host) && !downloadedUris.contains(it) }
+    }
+
     private fun isHostVisitable(host: String): Boolean {
         return host == spongeConfig.spongeUri.host ||
             spongeConfig.includeSubdomains && host.endsWith(spongeConfig.spongeUri.host)
